@@ -1,11 +1,11 @@
 ;;; jde-bug.el -- JDEbug Interface
-;; $Revision: 1.1 $ $Date: 2000/08/13 13:43:57 $ 
+;; $Revision: 1.2 $ $Date: 2001/02/12 05:38:22 $ 
 
 ;; Author: Paul Kinnucan <paulk@mathworks.com>
 ;; Maintainer: Paul Kinnucan
 ;; Keywords: java, tools
 
-;; Copyright (C) 1997, 1998, 1999 Paul Kinnucan.
+;; Copyright (C) 1997, 1998, 1999, 2000 Paul Kinnucan.
 
 ;; GNU Emacs is free software; you can redistribute it and/or modify
 ;; it under the terms of the GNU General Public License as published by
@@ -134,7 +134,11 @@ specifiy the options  -Xnoagent and -Djava.compiler=NONE."
 
 (defcustom jde-bug-debugger-host-address (if jde-xemacsp (system-name) system-name)
   "*Address of system on which JDEbug is running.
-The default value is the value of the standard Emacs variable `system-name'."
+The default value is the value of the standard Emacs variable `system-name'.
+The JDE uses the host address to connect to JDEBug during startup. On some Windows
+systems, the JDE is unable to connect to the debugger socket under the system name.
+If this happens, you can try setting this variable to the absolute address of 
+a local host: 127.0.0.1 ."
   :group 'jde-bug
   :type 'string)
 
@@ -199,6 +203,9 @@ breakpoint marker."
 	["Step Into"                  jde-bug-step-into 
 	                              (jde-dbs-target-process-steppable-p)]
 
+	["Step Into All"              jde-bug-step-into-all 
+	                              (jde-dbs-target-process-steppable-p)]
+
 	["Step Out"                   jde-bug-step-out 
 	                              (jde-dbs-target-process-steppable-p)]
 
@@ -208,9 +215,9 @@ breakpoint marker."
 	["Exit Debugger"              jde-bug-exit 
 	                              (jde-dbs-debugger-running-p)]
 	"-"
-	["Set Breakpoint"             jde-bug-set-breakpoint t]
+	;; Added by lea
+        ["Toggle Breakpoint"          jde-bug-toggle-breakpoint t]
 	["Set Conditional Breakpoint" jde-bug-set-conditional-breakpoint nil]
-	["Clear Breakpoint"           jde-bug-clear-breakpoint t]
 	["Save Breakpoints"           jde-bug-save-breakpoints nil]
 
 	(list
@@ -495,6 +502,8 @@ breakpoint marker."
 	 )
 	"-"
 	["Preferences"                jde-bug-show-preferences t]
+	"-"
+	["Help"                       jde-bug-help t]
 	)
 "Defines the JDE's debug menu.")
 
@@ -509,12 +518,12 @@ breakpoint marker."
 (make-variable-buffer-local 'jde-bug-minor-mode-p)
 
 
-(let ((a (assoc 'jde-bug-minor-mode-p minor-mode-map-alist)))
-  (if a
-      (setcdr a jde-bug-keymap)
-    (add-to-list 'minor-mode-map-alist
-		 (cons 'jde-bug-minor-mode-p
-		       jde-bug-keymap))))
+; (let ((a (assoc 'jde-bug-minor-mode-p minor-mode-map-alist)))
+;   (if a
+;       (setcdr a jde-bug-keymap)
+;     (add-to-list 'minor-mode-map-alist
+; 		 (cons 'jde-bug-minor-mode-p
+; 		       jde-bug-keymap))))
 
 (defun jde-bug-install-jdebug-menu ()
   "Installs the jdebug menu in the current Java source buffer
@@ -524,16 +533,23 @@ project."
        (or 
 	(not jde-xemacsp) 
 	(featurep 'infodock)))
-      (setq jde-bug-minor-mode-p t)
+      (progn
+	(setq jde-bug-minor-mode-p t)
+	(let ((a (assoc 'jde-bug-minor-mode-p minor-mode-map-alist)))
+	  (if a
+	      (setcdr a jde-bug-keymap)
+	    (add-to-list 'minor-mode-map-alist
+			 (cons 'jde-bug-minor-mode-p
+			       jde-bug-keymap)))))
     (if (and 
 	 (not (featurep 'infodock))
 	 (not (memq 'infodock c-emacs-features))
 	 (boundp 'current-menubar)
 	 current-menubar
 	 (not (car (find-menu-item current-menubar '("JDEbug")))))
-      (if (fboundp 'add-submenu)
-	  (add-submenu nil jde-bug-menu)
-	(add-menu nil "JDEbug" (cdr jde-bug-menu))))))
+	(if (fboundp 'add-submenu)
+	    (add-submenu nil jde-bug-menu)
+	  (add-menu nil "JDEbug" (cdr jde-bug-menu))))))
 
 (defun jde-bug-remove-jdebug-menu ()
   "Removes the jdebug menu from the menubar of the current Java source buffer."
@@ -554,9 +570,10 @@ project."
 (defcustom jde-bug-key-bindings
   (list (cons "[?\C-c ?\C-z ?\C-s]" 'jde-bug-step-over)
 	(cons "[?\C-c ?\C-z ?\C-x]" 'jde-bug-step-into)
+	(cons "[?\C-c ?\C-z ?\C-a]" 'jde-bug-step-into-all)
 	(cons "[?\C-c ?\C-z ?\C-w]" 'jde-bug-step-out)
 	(cons "[?\C-c ?\C-z ?\C-c]" 'jde-bug-continue)
-	(cons "[?\C-c ?\C-z ?\C-b]" 'jde-bug-set-breakpoint))
+	(cons "[?\C-c ?\C-z ?\C-b]" 'jde-bug-toggle-breakpoint))
   "*Specifies key bindings for JDEbug.
 The value of this variable is an association list. The car of
 each element specifies a key sequence. The cdr specifies 
@@ -717,13 +734,22 @@ You can use the notation [f1], [f2], etc., to specify function keys."
 
 
 (defun jde-bug-step-into () 
-  "Advances the process into the function invoked at point."
+  "Advances to the next step in the method at point except if the method
+   belongs to the java, javax, or sun packages."
   (interactive)
   (let* ((process (jde-dbs-get-target-process))
 	 (cmd
 	  (jde-dbs-step "step into" :process process :step-type "into")))
     (jde-dbs-cmd-exec cmd)))
 
+(defun jde-bug-step-into-all () 
+  "Advances the process into the function invoked at point."
+  (interactive)
+  (let* ((process (jde-dbs-get-target-process))
+	 (cmd
+	  (jde-dbs-step "step into" :process process :step-type "into-all")))
+    (jde-dbs-cmd-exec cmd)))
+
 (defun jde-bug-step-out () 
   "Advances the process to the next line in the invoking method."
   (interactive)
@@ -770,7 +796,6 @@ You can use the notation [f1], [f2], etc., to specify function keys."
 	(1+ ln)
       ln)))
 
-
 (add-hook 
  'jde-mode-hook 
  (lambda ()
@@ -794,8 +819,9 @@ You can use the notation [f1], [f2], etc., to specify function keys."
 		  (jde-bug-highlight-breakpoint line))))
 	  jde-bug-saved-breakpoints)))))
 
+
 (defun jde-bug-set-breakpoint()
-"Sets a breakpoint at the current line in the current buffer."
+  "Sets a breakpoint at the current line in the current buffer."
   (interactive)
   (let* ((file (file-name-nondirectory (buffer-file-name)))
          (line (jde-bug-get-line-at-point))
@@ -816,9 +842,22 @@ You can use the notation [f1], [f2], etc., to specify function keys."
                                 "set breakpoint" 
                                 :process proc
                                 :breakpoint bp))
-              (result (jde-dbs-cmd-exec set-breakpoint)))
+	       (result (jde-dbs-cmd-exec set-breakpoint)))
           (message "Breakpoint set at line %d in class %s." line file)))))
 
+;; test by lea
+
+(defun jde-bug-toggle-breakpoint ()
+  "Toggles the breakpoint on the current line."
+  (interactive)
+  (let*  ((file (file-name-nondirectory (buffer-file-name)))
+	  (line (jde-bug-get-line-at-point))
+	  (bp (jde-bug-breakpoints-find file line)))
+    (if bp
+	(jde-bug-clear-breakpoint)
+      (jde-bug-set-breakpoint))))
+
+
 (defun jde-bug-set-conditional-breakpoint () 
   (interactive)
   (message "not implemented"))
@@ -829,11 +868,12 @@ You can use the notation [f1], [f2], etc., to specify function keys."
 ;;                                                                            ;;
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
+
 (defun jde-bug-clear-breakpoint()
   "Clear the breakpoint at the current line in the current buffer."
   (interactive)
   (let* ((file (file-name-nondirectory (buffer-file-name)))
-	 (line (count-lines (point-min) (point)))
+	 (line (jde-bug-get-line-at-point))
 	 (bp (jde-bug-breakpoints-find file line))
 	 (proc (jde-dbs-get-target-process)))
     (if (and bp proc)
@@ -847,15 +887,22 @@ You can use the notation [f1], [f2], etc., to specify function keys."
 	(jde-bug-breakpoints-remove bp))
     (jde-bug-remove-breakpoint-highlight)))
 
+
 (defun jde-bug-remove-breakpoint-highlight ()
   (if jde-xemacsp
-      (let ((highlight (extent-at (line-beginning-position))))
-	(when highlight
-	  (delete-extent highlight)))
-    (let ((highlight (car (overlays-at (line-beginning-position)))))
-      (when highlight
-	(delete-overlay highlight)))))
-
+      (map-extents
+       (lambda (extent maparg)
+	 (delete-extent extent))
+       (current-buffer)
+       (line-beginning-position) (line-end-position) 
+       nil nil
+       'face 'jde-bug-breakpoint-marker)     
+    (let ((overlays (overlays-at (line-beginning-position))))
+      (while overlays
+	(let ((overlay (car overlays)))
+	  (if (eq (overlay-get overlay 'face) 'jde-bug-breakpoint-marker)
+		  (delete-overlay overlay)))
+	(setq overlays (cdr overlays))))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;                                                                            ;;
@@ -2501,7 +2548,7 @@ pane of the JDEBug's three-pane window configuration."
 
 ;;;###autoload
 (defun jde-bug-debug-app ()
-  "*Runs the debugger on the application in the current source buffer."
+  "Runs the debugger on the application in the current source buffer."
   (interactive)
   (if (and 
        (jde-bug-jpda-installed-p)
@@ -2514,11 +2561,55 @@ pane of the JDEBug's three-pane window configuration."
 	      (jde-bug-set-breakpoints process jde-bug-breakpoints)
 	      (setq result (jde-bug-continue)))))))
 
+(defun jde-bug-help ()
+  "Displays the JDEbug User's Guide."
+  (interactive)
+  (let* ((jde-dir (jde-find-jde-doc-directory))
+         (jdebug-help
+          (if jde-dir
+              (if (and jde-xemacsp
+                       (locate-data-directory "jde"))
+                  (expand-file-name "jdebug-ug.html" jde-dir)
+                (expand-file-name "doc/html/jdebug-ug/jdebug-ug.html" jde-dir)))))       
+    (if (and
+         jdebug-help
+         (file-exists-p jdebug-help))
+        (browse-url (concat "file://" (jde-convert-cygwin-path jdebug-help))
+                    browse-url-new-window-p)
+      (signal 'error '("Cannot find JDEbug User's Guide.")))))
+
+(defun jde-bug-keys ()
+  "Displays JDEbug keybindings. Use `jde-keys' to display JDE keybindings."
+  (interactive)
+  (jde-describe-map 'jde-bug-keymap))
+
+
 (provide 'jde-bug)
 
 ;; $Log: jde-bug.el,v $
-;; Revision 1.1  2000/08/13 13:43:57  michaels
-;; Initial checkin.
+;; Revision 1.2  2001/02/12 05:38:22  paulk
+;; JDE 2.2.7
+;;
+;; Revision 1.57  2000/12/18 05:22:45  paulk
+;; *** empty log message ***
+;;
+;; Revision 1.56  2000/11/18 05:31:43  paulk
+;; Fixed bug in jde-bug-remove-breakpoint-highlight.
+;;
+;; Revision 1.55  2000/11/16 02:58:29  paulk
+;; Fixed bug that caused highlight not to be removed from breakpint.
+;;
+;; Revision 1.54  2000/09/23 04:33:26  paulk
+;; Fixed bug in jde-bug-clear-breakpoint command where the command was off by a line. Thanks to Stan Lanning <lanning@pobox.com> for providing this fix.
+;;
+;; Revision 1.53  2000/08/14 02:33:36  paulk
+;; Adds support for Step Into All command.
+;;
+;; Revision 1.52  2000/07/28 06:27:44  paulk
+;; Committing all modified files.
+;;
+;; Revision 1.51  2000/07/13 05:22:46  paulk
+;; *** empty log message ***
 ;;
 ;; Revision 1.50  2000/06/12 08:29:32  paulk
 ;; Restored jde-bug-install-jdebug-menu for XEmacs compatibility.
