@@ -1,5 +1,5 @@
 ;;; jde-make.el -- Integrated Development Environment for Java.
-;; $Revision: 1.1 $ 
+;; $Revision: 1.2 $ 
 
 ;; Author: Paul Kinnucan <paulk@mathworks.com>
 ;; Maintainer: Paul Kinnucan
@@ -29,6 +29,13 @@
  :group 'jde-project
  :type 'string)
 
+(defcustom jde-make-working-directory ""
+  "*Path of the working directory to use in 'make' build mode.
+If this string is empty, the 'make' build mode uses the current file
+location as its working directory"
+  :group 'jde-project
+  :type 'string)
+
 (defcustom jde-make-args ""
   "*Specifies arguments to be passed to make program."
   :group 'jde-project
@@ -72,7 +79,12 @@ list of arguments entered in the minibuffer."
 
   (let ((make-command
 	 (jde-make-make-command 
-	  jde-interactive-make-args)))
+	  jde-interactive-make-args))
+	(default-directory 
+	  (if (string= jde-make-working-directory "")
+	      default-directory
+	    jde-make-working-directory)))
+
     ;; Force save-some-buffers to use the minibuffer
     ;; to query user about whether to save modified buffers.
     ;; Otherwise, when user invokes jde-make from
@@ -109,8 +121,11 @@ list of arguments entered in the minibuffer."
 (provide 'jde-make)
 
 ;; $Log: jde-make.el,v $
-;; Revision 1.1  2000/08/13 13:44:00  michaels
-;; Initial checkin.
+;; Revision 1.2  2001/02/12 05:38:26  paulk
+;; JDE 2.2.7
+;;
+;; Revision 1.7  2000/08/09 03:29:26  paulk
+;; Added jde-make-working-directory variable. Thanks to Laurent Latil <Laurent.Latil@france.sun.com>
 ;;
 ;; Revision 1.6  1999/04/27 16:44:49  paulk
 ;; Updated to allow interactive entry of make arguments. Thanks to Yarek J. Kowalik <jgk@klg.com> for providing this enhancement.
