@@ -1,11 +1,11 @@
 ;; jde-run.el --- runs the Java app in the current buffer.
-;; $Revision: 1.2 $ $Date: 2000/10/06 08:51:20 $
+;; $Revision: 1.3 $ $Date: 2001/02/12 05:38:26 $
 
 ;; Author: Paul Kinnucan <paulk@mathworks.com>
 ;; Maintainer: Paul Kinnucan
 ;; Keywords: tools, processes
 
-;; Copyright (C) 1997, 1998 Paul Kinnucan
+;; Copyright (C) 1997, 1998, 1999, 2000, 2001 Paul Kinnucan
 
 ;; GNU Emacs is free software; you can redistribute it and/or modify
 ;; it under the terms of the GNU General Public License as published by
@@ -29,7 +29,7 @@
 ;; JDE User's Guide for more information.
 
 ;; The latest version of the JDE is available at
-;; <URL:http://sunsite.auc.dk/jde/>.
+;; <URL:http://jde.sunsite.dk>.
 
 ;; Please send any comments, bugs, or upgrade requests to
 ;; Paul Kinnucan at paulk@mathworks.com.
@@ -55,13 +55,13 @@ method."
 If you specify a path, the JDE launches the application from the
 directory specified by the path."
   :group 'jde-project
-  :type 'string)
+  :type 'file)
 
 
 (defcustom jde-run-java-vm "java"
   "*Specify Java interpreter for non-Windows platforms."
   :group 'jde-project
-  :type 'string)
+  :type 'file)
 
 (defcustom jde-run-java-vm-w "javaw"
   "*Specify Java interpreter for Windows platforms.
@@ -71,7 +71,7 @@ the javaw version of the JDK Java interpreter to prevent a console
 window from appearing every time you run an application. Sound
 confusing? It is, but it works."
   :group 'jde-project
-  :type 'string)
+  :type 'file)
 
 
 (defcustom jde-run-classic-mode-vm nil
@@ -236,14 +236,14 @@ the command line."
 (defcustom jde-run-applet-viewer ""
   "*Specify name of viewer to use to display page containing the applet."
   :group 'jde-project
-  :type 'string)
+  :type 'file)
 
 (defcustom jde-run-applet-doc ""
   "*Specify name of document containing applet to be viewed.
 If no document is specified, JDE assumes that the document name is
 APPLET.html, where APPLET is the name of the applet to be viewed."
   :group 'jde-project
-  :type 'string)
+  :type 'file)
 
 
 (defcustom jde-appletviewer-option-encoding ""
@@ -265,8 +265,7 @@ virtual machine that runs appletviewer."
 If you do not specify an executable, the JDE runs the vm specified
 by `jde-run-java-vm' or `jde-run-java-vm-w'."
   :group 'jde-project
-  :type 'string
-)
+  :type 'file)
 
 (defcustom jde-run-executable-args nil
   "*Specify arguments to be passed to the application executable.
@@ -363,20 +362,6 @@ panel to specifying the applet document."
       (setq jde-run-applet-doc nil)
     (setq jde-run-applet-doc doc)))
 
-(defun jde-run-build-classpath-arg (path-list)
-"Build a classpath from a list of paths."
-  (let ((classpath "")
-	(len (length path-list))
-	(n 0))
-    (while (< n len)
-      (setq classpath (concat classpath
-			      (if (> n 0)
-				  jde-classpath-separator)
-			      (elt path-list n)))
-      (setq n (1+ n)))
-      (jde-convert-cygwin-path classpath jde-classpath-separator)))
-
-
 (defun jde-run-get-vm-args ()
   "Builds a command-line argument string to pass to the Java vm.
 This function builds the string from the values of the JDE
@@ -396,18 +381,16 @@ Run Option panel variables."
     ;; Set the classpath option. Use the local
     ;; classpath, if set; otherwise, the global
     ;; classpath.
-    (if jde-run-option-classpath
-	(setq options
-	      (nconc options 
-		     (list "-classpath"
-			   (jde-run-build-classpath-arg
-			    jde-run-option-classpath))))
-      (if jde-global-classpath
-	  (setq options
-		(nconc options
-		       (list "-classpath"
-			     (jde-run-build-classpath-arg
-			      jde-global-classpath))))))
+    (setq options
+	  (nconc
+	   options
+	   (list
+	    "-classpath"
+	    (jde-build-classpath
+	     (if jde-run-option-classpath
+		 jde-run-option-classpath
+	       jde-global-classpath)))))
+    
 
     ;; Set the verbose options.
     (let ((print-classes-loaded
@@ -540,9 +523,11 @@ not the null string. Otherwise run the class specified by
 the current buffer. This command also creates a comint buffer to allow
 you to interacti with the program."
   (interactive)
-  (if (string= jde-run-executable "")
-      (jde-run-main-class)
-    (jde-run-executable)))
+  (if (string= mode-name "JDE")
+      (if (string= jde-run-executable "")
+	  (jde-run-main-class)
+	(jde-run-executable))
+    (error "The jde-run command works only in a Java source buffer.")))
 
 (defun jde-run-get-main-class () 
   "Gets the main class for the application to which the current
@@ -566,20 +551,13 @@ input from the buffer to the program."
   (interactive)
     (jde-run-internal (jde-run-get-main-class)))
 
+
 (defmacro save-w32-show-window (&rest body)
   "Saves the value of the w32-start-process-show-window variable
 before evaluating body and restores the value afterwards."
-  `(if (and (eq system-type 'windows-nt)
-	    (not jde-xemacsp))
-       (if (boundp 'win32-start-process-show-window)
-	   (let ((save win32-start-process-show-window))
-	     (setq win32-start-process-show-window t)
-	     ,@body
-	     (setq win32-start-process-show-window save))
-	 (let ((save w32-start-process-show-window))
-	   (setq w32-start-process-show-window t)
-	   ,@body
-	   (setq w32-start-process-show-window save)))	 
+  `(let ((win32-start-process-show-window t)
+	 (w32-start-process-show-window t)
+	 (windowed-process-io t))		 
      ,@body))
 
 (defun jde-run-unquote (string)
@@ -637,8 +615,12 @@ buffer belongs is running."
 	    (insert (concat "cd " working-directory "\n"))
 	    (insert command-string)
 	    (jde-run-mode))
-	  (save-w32-show-window
-	   (comint-exec run-buffer app-class prog nil prog-args))
+; 	  (save-w32-show-window
+; 	   (comint-exec run-buffer app-class prog nil prog-args))
+	  (let ((win32-start-process-show-window t)
+		(w32-start-process-show-window t)
+		(windowed-process-io t))
+	    (comint-exec run-buffer app-class prog nil prog-args))
 	  (pop-to-buffer run-buffer)
 	  (cd source-directory))
       (message "An instance of %s is running." app-class)
@@ -758,16 +740,18 @@ under Windows."
     (apply 'start-process-shell-command name buffer command switches)))
 
 (defun jde-run-applet-internal (doc)
-  (let* ((doc-name (file-name-sans-extension
-			(file-name-nondirectory doc)))
+  (let* ((doc-file-name (file-name-nondirectory doc))
+	 (doc-directory (file-name-directory doc))
+	 (doc-name (file-name-sans-extension doc-file-name))
 	 (run-buf-name (concat "*" doc-name "*")))
+
     (if (not (comint-check-proc run-buf-name))
 	(let* ((run-buffer (get-buffer-create run-buf-name))
 	       (win32-p (eq system-type 'windows-nt))
 	       (prog jde-run-applet-viewer)
 	       (prog-args
 		(append (jde-get-appletviewer-options)
-			(list doc)))
+			(list doc-file-name)))
 	       (command-string (concat prog " "
 				       (jde-run-make-arg-string
 					prog-args)
@@ -775,7 +759,8 @@ under Windows."
 	  (save-excursion
 	    (set-buffer run-buffer)
 	    (erase-buffer)
-	    (insert (concat "cd " default-directory "\n"))
+	    (cd doc-directory)
+	    (insert (concat "cd " doc-directory "\n"))
 	    (insert command-string)
 	    (jde-run-mode))
 	  (jde-run-applet-exec run-buffer doc-name prog nil prog-args)
@@ -807,7 +792,7 @@ directory."
 
 
 
-(setq jde-run-applet-document nil) 
+(setq jde-run-applet-last-doc nil) 
 
 ;;;###autoload
 (defun jde-run-applet (&optional doc)
@@ -828,11 +813,11 @@ second file. Otherwise, this command will simply redisplay the first
 file."
   (interactive
    (let ((insert-default-directory nil))
-     (list (read-file-name "Applet doc: " nil nil nil jde-run-applet-document))))
-  (setq jde-run-applet-document doc)
-  (let ((applet-doc (if (and jde-run-applet-document
-			     (not (string= jde-run-applet-document "")))
-			jde-run-applet-document
+     (list (read-file-name "Applet doc: " nil nil nil jde-run-applet-last-doc))))
+  (setq jde-run-applet-last-doc doc)
+  (let ((applet-doc (if (and jde-run-applet-last-doc
+			     (not (string= jde-run-applet-last-doc "")))
+			jde-run-applet-last-doc
 		      (if (and jde-run-applet-doc
 			       (not (string= jde-run-applet-doc "")))
 			    jde-run-applet-doc
@@ -858,11 +843,29 @@ file."
 
 ;; Change History
 ;; $Log: jde-run.el,v $
-;; Revision 1.2  2000/10/06 08:51:20  youngs
-;; Martin's Monster Mega typo patch
+;; Revision 1.3  2001/02/12 05:38:26  paulk
+;; JDE 2.2.7
 ;;
-;; Revision 1.1  2000/08/13 13:44:01  michaels
-;; Initial checkin.
+;; Revision 1.43  2001/02/03 08:23:53  paulk
+;; Changed declaration of customized variables so you can use completion on path variables.
+;;
+;; Revision 1.42  2001/02/03 07:30:27  paulk
+;; Now uses jde-build-classpath instead of jde-run-build-classpath to build classpath argument.
+;;
+;; Revision 1.41  2001/02/01 06:13:47  paulk
+;; *** empty log message ***
+;;
+;; Revision 1.40  2000/10/20 04:11:07  paulk
+;; Fix that allows the JDE to be used with NT/XEmacs.
+;;
+;; Revision 1.39  2000/10/08 12:55:39  paulk
+;; *** empty log message ***
+;;
+;; Revision 1.38  2000/09/21 04:45:32  paulk
+;; Updates jde-run-applet to work with the appletviewer in JDK 1.3.
+;;
+;; Revision 1.37  2000/08/19 07:02:02  paulk
+;; Changed variable name.
 ;;
 ;; Revision 1.36  2000/03/16 05:18:11  paulk
 ;; Miscellaneous small bug fixes and enhancements.
@@ -1033,4 +1036,4 @@ file."
 ;; Initial revision
 ;;
 
-;;; jde-run.el ends here
+;;; jde-run.el ends here
\ No newline at end of file
