@@ -1,11 +1,11 @@
 ;;; jde-widgets.el -- Custom-style widgets used by the JDE
-;; $Revision: 1.1 $ $Date: 2000/08/13 13:44:01 $ 
+;; $Revision: 1.3 $ $Date: 2001/02/17 07:38:21 $ 
 
 ;; Author: Paul Kinnucan <paulk@mathworks.com>
 ;; Maintainer: Paul Kinnucan
 ;; Keywords: java, tools
 
-;; Copyright (C) 1997, 1998, 1999 Paul Kinnucan.
+;; Copyright (C) 1997, 1998, 1999, 2001 Paul Kinnucan.
 
 ;; GNU Emacs is free software; you can redistribute it and/or modify
 ;; it under the terms of the GNU General Public License as published by
@@ -39,6 +39,7 @@
 
 (require 'wid-edit)
 (require 'eieio)
+(require 'cl)
 
 
 ;; ----------------------------------------------------------------------
@@ -102,7 +103,8 @@
 	   (insert (widget-get widget :prefix-empty) tag "\n"))
 	  (open
 	   ;; Open node.
-	   (push (widget-create-child-and-convert widget 'jde-widget-tree-close-button)
+	   (push 
+	    (widget-create-child-and-convert widget 'jde-widget-tree-close-button)
 		buttons)
 	   (insert "-\\ " tag "\n")
 	   (let ((prefix (concat (widget-get widget :prefix)
@@ -214,6 +216,8 @@
     "Ok")
    :dialog this)
 
+  (widget-insert "  ")
+
   (widget-put
    (widget-create 
     'push-button
@@ -478,11 +482,146 @@ expand button causes the widget to display the values of the array."
   (widget-setup))
 
 
+
+;; ----------------------------------------------------------------------
+;; Option Tree Widget
+
+(defun jde-widget-option-tree-open-button-callback (widget &optional event)
+  ;; Set parent state to open.
+  (widget-value-set (widget-get widget :parent) t)
+  (widget-setup))
+
+(define-widget 'jde-widget-option-tree-open-button 'item
+  "Button to open an option tree."
+  :button-prefix "["
+  :button-suffix "]"
+  :tag "+"
+  :action 'jde-widget-option-tree-open-button-callback
+  :help-echo "Show option tree."
+  :format "%[%t%]")
+
+(define-widget 'jde-widget-option-tree-close-button 'item
+  "Close node in `jde-widget-option-tree' widget."
+  :button-prefix "["
+  :button-suffix "]"
+  :tag "-"
+  :action 'jde-widget-option-tree-close-button-callback
+  :help-echo "Hide panel."
+  :format "%[%t%]")
+
+(defun jde-widget-option-tree-close-button-callback (widget &optional event)
+  ;; Set parent state to closed.
+  (let* ((parent (widget-get widget :parent))
+	(entries (widget-get parent :args))
+	(group (car (widget-get parent :children)))
+	(children (widget-get group :children)))
+    ;; Get values entered by user from children and
+    ;; insert them in the corresponding widget definitions
+    ;; so that they appear the next time the user expands
+    ;; the tree.
+    (while (and entries children)
+      (widget-put (car entries) :value (widget-value (car children)))
+      (setq entries (cdr entries)
+	    children (cdr children)))
+    (widget-value-set parent nil)))
+
+(defun jde-widget-option-tree-value-create-callback (widget)
+  (let ((open-widget-p (widget-value widget))
+	(tag (widget-get widget :tag))
+	(entries (widget-get widget :args))
+	entry children buttons)
+    (cond (open-widget-p
+	   ;; Wrap widgets in this tree in a group widget
+           ;; to ensure proper formatting.
+	   (let ((group-type
+		  (list 'group :args entries))
+		 group-widget) 
+	   (push 
+	    (widget-create-child-and-convert 
+	     widget 
+	     'jde-widget-option-tree-close-button)
+		buttons)
+	   (insert "-\\ " tag "\n")	   
+	   (push (widget-create-child-and-convert widget group-type) children)))
+	  (t
+	   (push (widget-create 'jde-widget-option-tree-open-button 
+			  :parent widget)
+		 buttons)
+	   (insert "-- " tag "\n")))
+    (widget-put widget :children children)
+    (widget-put widget :buttons buttons)))
+    
+
+(define-widget 'jde-widget-option-tree 'default
+  "A panel containing widgets."
+  :convert-widget 'widget-types-convert-widget
+  :format "%v"
+  :value-get 'widget-value-value-get
+  :value-create 'jde-widget-option-tree-value-create-callback
+  :value-delete 'widget-children-value-delete)
+
+
+(defun test-option-tree ()
+  (interactive)
+  (switch-to-buffer "*Panel Example*")  
+  (kill-all-local-variables)
+  (let ((inhibit-read-only t))
+    (erase-buffer))  
+  (let ((all (overlay-lists)))  
+    (mapcar 'delete-overlay (car all))    
+    (mapcar 'delete-overlay (cdr all))) 
+
+  (widget-insert "Test panel widget. \n\n")
+
+  (let ((panel (widget-create 
+		'jde-widget-option-tree
+		:tag "Compile Options"
+		'(cons :tag "Debugger Options"
+		   (radio-button-choice :format "%t \n%v"
+					:tag "Debugger "
+					(const "JDEbug")
+					(const "jdb")
+					(const "oldjdb")
+					(const "Other"))
+		   (cons :tag "Other Debugger Info"
+			 (string :tag "Path")
+			 (radio-button-choice :format "%t \n%v"
+					      :tag "Type "
+					      (const "Executable")
+					      (const "Class"))))
+		  '(repeat (string :tag "Path"))
+		  '(editable-field :tag "classpath" 
+				  :format "  %t:  %v\n  %h \n\n"
+				  :size 40
+				  :doc "Name of project.")
+		  '(editable-field :tag "compiler"
+				  :format "  %t:  %v\n  %h \n\n"
+				  :size 40
+				  :doc "Name of project.")
+		  '(jde-widget-option-tree :tag "Debugger Options"
+					   (repeat (string :tag "Path"))))))
+    (use-local-map widget-keymap)
+    (widget-setup)))
+
+
+
 (provide 'jde-widgets)
 
 ;; $Log: jde-widgets.el,v $
-;; Revision 1.1  2000/08/13 13:44:01  michaels
-;; Initial checkin.
+;; Revision 1.3  2001/02/17 07:38:21  paulk
+;; Requires cl to enable batch compile.
+;;
+;; Revision 1.10  2001/02/16 04:38:52  paulk
+;; Added (require 'cl) to enable batch compile.
+;;
+;; Revision 1.9  2000/12/18 05:22:46  paulk
+;; *** empty log message ***
+;;
+;; Revision 1.8  2000/11/27 06:15:02  paulk
+;; Added an experimental jde-widget-option-tree widget.
+;;
+;; Revision 1.7  2000/08/31 05:22:17  paulk
+;; Fixed bug on XEmacs where the Cancel button overwrote the OK button in the standard dialog class.
 ;;
 ;; Revision 1.6  2000/02/01 04:11:57  paulk
 ;; ReleaseNotes.txt
