@@ -1,11 +1,11 @@
 ;;; jde-wiz.el
-;; $Revision: 1.1 $ 
 
+;; $Revision: 1.3 $
 ;; Author: Paul Kinnucan <paulk@mathworks.com>
 ;; Maintainer: Paul Kinnucan
 ;; Keywords: java, tools
 
-;; Copyright (C) 1997, 1998 Paul Kinnucan.
+;; Copyright (C) 1997, 1998, 2000 Paul Kinnucan.
 
 ;; GNU Emacs is free software; you can redistribute it and/or modify
 ;; it under the terms of the GNU General Public License as published by
@@ -23,40 +23,37 @@
 ;; Boston, MA 02111-1307, USA.
 
 (require 'beanshell)
+(require 'jde-complete)
 
 
-(defcustom jde-wiz-insert-excluded-packages-regexp "bsh.*"
-  "*Specifies classes that should not be imported into a source file.
-The value of this variable should be a regular expression. The `jde-wiz-find-and-import' command does not import any classes whose fully qualified names matchthe regular expression. If more than one fully qualified class name matches theunqualified name that you specify, the command prompts you to select only the classes that do not match the regular expression."
-  :group 'jde-project
-  :type 'string)
+
+;;
+
+
+(defun jde-wiz-escape-backslashes-in-path (path) 
+  (mapconcat
+   (lambda (char) 
+     (if (eq char ?\\) "\\\\" (char-to-string char)))
+   path ""))
 
 (defun jde-wiz-update-class-list()
   "Update the class list used to resolve class names.
 The first time you invoke a JDE wizard, the JDE builds a list of all classes on the classpath
 defined by jde-global-classpath. Wizards use this list to resolve unqualified class names. If you add any classes to the classpath after invoking a wizard, you should update the class list."
   (interactive)
-  (message "Rescanning classes..." )
-  (bsh-eval "jde.util.JdeUtilities.buildClassList();" )
-  (message "Rescanning classes...Complete"))
+  (when (get-process "bsh")
+    (message "Rescanning classes...")
+    (bsh-eval (concat "jde.util.JdeUtilities.buildClassList("
+		      (jde-wiz-escape-backslashes-in-path
+		       (jde-build-path-arg nil jde-global-classpath t)
+)
+                    ");"))
+    (message "Rescanning classes...Complete"))
+)
 
 
-
-(defun jde-wiz-get-imports()
-  (let ((import-re "import[ ]+\\(.*\\)[ ]*;")
-	(imports nil))    
-    (save-excursion
-      (goto-char (point-max))
-      (while (re-search-backward import-re (point-min) t)
-	(looking-at import-re)
-	(setq imports (nconc imports 
-			     (list (buffer-substring-no-properties 
-				    (match-beginning 1) 
-				    (match-end 1)))))))
-    imports))
-
 (defun jde-wiz-get-package-name ()
-  (let ((package-re "package[ \t]+\\(.*\\)[ \t]*;"))
+  (let ((package-re "^[ \t]*package[ \t]+\\(.*\\)[ \t]*;"))
     (save-excursion
       (goto-char (point-max))
       (when (re-search-backward package-re (point-min) t)
@@ -65,233 +62,6 @@ defined by jde-global-classpath. Wizards use this list to resolve unqualified cl
 		       (match-beginning 1)
 		       (match-end 1))))))
 
-(defun jde-wiz-get-import-insertion-point ()
-  (let ((ip-re
-	 (list (cons "import[ ]+\\(.*\\)[ ]*;" 'backward)
-	       (cons "package[ \t]+\\(.*\\)[ \t]*;" 'backward)
-	       (cons "^$" 'forward)))
-	insertion-point n)
-    (save-excursion
-      (let ((i 0)
-	    (n (length ip-re)))
-	(while (and
-		(not insertion-point)
-		(< i n))
-	  (let ((re (car (nth i ip-re)))
-		(direction (cdr (nth i ip-re))))
-	    (if (eq direction 'forward)
-		(progn
-		  (goto-char (point-min))
-		  (setq insertion-point (re-search-forward re (point-max) t)))
-	      (goto-char (point-max))
-	      (setq insertion-point (re-search-backward re (point-min) t)))
-	    (when insertion-point
-	      (forward-line 1)
-	      (setq insertion-point (point))))
-	  (setq i (+ i 1)))))
-    insertion-point))
-
-
-(defun jde-wiz-import (class) 
-  "*Insert an import statement for a class in the current buffer.
-CLASS is the fully qualified name of the class to be imported. This
-function allows you to enter an import at the head of your buffer
-from any point in the buffer. The function does nothing if an import
-statement for the specified class alrady exists."
-  (interactive
-   "sClass: ")
-  (jde-wiz-insert-imports (list class)))
-
-;; Contributed by David Ponce <david_ponce@mail.schneider.fr>
-(defun jde-sort-imports (&optional reverse)
-  "Sort Java import statements alphabetically. In reverse order if
-REVERSE is non-nil.
-
-Usage:
-  \\[jde-sort-imports] sort import statements ascending.
-  \\[universal-argument] \\[jde-sort-imports] sort descending.
-
-The the current buffer must be in `jde-mode'. This command uses the
-semantic Java parser and requires JDE 2.1.6-beta24 and above."
-  (interactive "P")
-  (or (eq major-mode 'jde-mode)
-      (error "Invalid major mode found. Must be 'jde-mode'."))
-  (or (and (local-variable-p 'semantic-toplevel-bovine-table (current-buffer))
-           (symbol-value 'semantic-toplevel-bovine-table))
-      (error "Semantic Java parser not found. JDE 2.1.6-beta24+
-needed."))
-  (and (interactive-p)
-       (consp current-prefix-arg)
-       (setq reverse t))
-  (let* ((tokens  (semantic-bovinate-toplevel nil nil t))
-         (depends (semantic-find-nonterminal-by-token 'include tokens)))
-    (if depends
-        (let* ((first-import-token (car depends))
-               (last-import-token  (nth (1- (length depends)) depends))
-               (start (semantic-token-start first-import-token))
-               (end   (semantic-token-end   last-import-token)))
-          (if (and start end)
-              (let (sort-fold-case)
-                (sort-lines reverse start end)
-                (goto-char start)))))))
-
-
-(defun jde-wiz-strip-package-from-class (class-name)
-  "Strips the package name from fully qualified java class"
-  (let (i return-name)
-    (setq return-name class-name)
-    (setq i (string-match "[^.]*$" class-name))
-    (if i
-        (setq return-name (substring class-name i)))
-    return-name))
-
-
-(defun jde-wiz-get-existing-import (class-name)
-  ""
-  (let ((import-re "import[ ]+\\(.*\\)[ ]*;")
-	(imports nil)
-        (existing-import)
-        (result nil))
-    (save-excursion
-      (goto-char (point-max))
-      (while (re-search-backward import-re (point-min) t)
-	(looking-at import-re)
-        (setq existing-import (buffer-substring-no-properties
-                               (match-beginning 1)
-                               (match-end 1)))
-        (if (string-equal class-name
-                          (jde-wiz-strip-package-from-class
-			   existing-import))
-            (setq result existing-import))))
-    result))
-
-;; Thanks to Jim LoVerde<loverde@str.com> for providing the code to 
-;; prevent duplicate imports.
-(defun jde-wiz-find-and-import (class)
-  "*Insert an import statement for a class in the current buffer.
-CLASS is an unqualified class name. This function inserts an import
-statement only if one does not already exist for the class.  This
-function searches the classpath for a class (or classes) that match
-CLASS. If it finds only one, it inserts an import statements for the
-class at the head of the current buffer. If it finds more than one
-class that matches CLASS, it prompts you to select which class to
-import. You can use the variable
-`jde-wiz-insert-excluded-packages-regexp' to prevent specified classes
-from being imported or consider for import. This command uses the
-JDE's BeanShell interpreter. It starts the interpreter if it is not
-already running so there may be a short delay generating the first
-import statement in the session. Note that you must explicitly include
-any directories or jars that you want the command to search in your
-classpath, including jars implicitly included by the jvm, e.g.,
-rt.jar."
-  (interactive
-   (list (read-from-minibuffer "Class: "
-			       (thing-at-point 'symbol))))
-  (let (existing-import)
-    (setq existing-import (jde-wiz-get-existing-import class))
-    (if (not (null existing-import))
-	(message "Skipping: already imported %s" existing-import)
-      (let ((imports 
-	     (bsh-eval-r 
-	      (concat "jde.util.JdeUtilities.getQualifiedName(\""
-		      class "\");"))))
-	(if imports
-	    (jde-wiz-insert-imports imports)
-	  (message "Error: could not find %s." class))))))
-					    
-		    
-(defun jde-wiz-insert-imports (new-imports) 
-  (let* ((imports
-	  (mapcar 'jde-wiz-strip-excluded-imports
-	   (jde-wiz-strip-existing-imports new-imports 
-					   (jde-wiz-get-imports)))))
-    ;;Delete the nil which result from the excluded ones
-    (setq imports (delq nil imports))
-    ;; If more than one class matches the specified class name, 
-    ;; prompt the user to select a class for import.
-    (if (> (length imports) 1 )
-	(jde-wiz-choose-imports imports)
-      (jde-wiz-insert-imports-into-buffer imports))))
-
-(defun jde-wiz-strip-excluded-imports (new-import)
-  "Removes excluded imports from the list"
-  ;;if the string matchs the regexp we want to ignore it. 
-  (if (string-match jde-wiz-insert-excluded-packages-regexp
-		    (concat " " new-import))
-      (progn (message "Excluding import: %s" new-import)
-	     ())
-    new-import))
-
-(defun jde-wiz-insert-imports-into-buffer (new-imports)
-  "Inserts imports into the correct place in the buffer."
-  (let (i n)
-    (save-excursion
-      (goto-char (jde-wiz-get-import-insertion-point))
-      (setq i 0)
-      (setq n (length new-imports))
-      (while (< i n)
-	(let ((new-import 
-	       (nth i new-imports)))
-	  (progn
-	    (insert
-	     (concat "import " new-import ";\n"))
-	    (message "Imported %s" new-import))
-	  (setq i (+ i 1)))))))
-
-
-(defun jde-wiz-strip-existing-imports (new-imports existing-imports)
-  "Exclude classes that have already been imported."
-  (let (i n return-imports)
-    (setq i 0)
-    (setq n (length new-imports))
-    (while (< i n)
-      ;;iterate through the new imports
-      (let((new-import
-	    (nth i new-imports)))
-	;;Strip out those alreay there
-	(when (not (find new-import existing-imports :test 'string=))
-	  (setq return-imports (nconc (list new-import)
-				      return-imports))))
-      (setq i(+ i 1)))
-    ;;Return any that still exist
-    return-imports))
-
-
-(defun jde-wiz-choose-imports (new-imports)
-  "Prompts the user to select a class to import from a list of similarly
- named candidates."
-  (let ((buf (get-buffer-create "*Select Import Class*" )))
-    (setq jde-wiz-import-window-config (current-window-configuration))
-    (setq jde-wiz-selected-import (car new-imports))
-    (set-buffer buf)
-    (widget-insert "Several classes match the name you specified.\n")
-    (widget-insert "Select the one you want to import.\n")
-    (widget-insert "Then click the OK button.\n" )
-    (let ((args (list
-		 'radio-button-choice
-		 :value (car new-imports)
-		 :notify (lambda (widget &rest ignore)
-			   (setq jde-wiz-selected-import (widget-value widget))
-			   (message "You selected: %s"
-				    (widget-value widget))))))
-      (setq args (nconc
-		  args
-		  (mapcar (lambda (x) (list 'item x)) new-imports)))
-      (apply 'widget-create args))
-    (widget-insert "\n")
-    (widget-create 'push-button
-		   :notify (lambda (&rest ignore)
-			     (let ((dialog-buffer
-				    (current-buffer)))
-			       (delete-window)
-			       (kill-buffer dialog-buffer)
-			       (set-window-configuration jde-wiz-import-window-config)
-			       (jde-wiz-insert-imports-into-buffer (cons jde-wiz-selected-import nil))
-			       (message "Import complete.")))
-		   "Ok")
-    (use-local-map widget-keymap)
-    (widget-setup)
-    (pop-to-buffer buf)))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;; Method Interface Implementation wizard                                  ;;
@@ -351,7 +121,7 @@ rt.jar."
 		    "jde.wizards.InterfaceFactory.getImportedClasses();")))
 	      (insert code)
 	      (if required-imports
-		  (jde-wiz-insert-imports-into-buffer required-imports))
+		  (jde-import-insert-imports-into-buffer required-imports))
 	      (jde-wiz-update-implements-clause interface-name))))	  
     (error
      (message "%s" (error-message-string err)))))
@@ -439,7 +209,7 @@ NOTE: this command works only if the overriding class
 	   "jde.wizards.MethodOverrideFactory.getImportedClasses();")))
     (insert skeleton)
     (if required-imports
-	(jde-wiz-insert-imports required-imports))))
+	(jde-import-insert-imports required-imports))))
 
 
 (defun jde-wiz-override-variant-method (methods) 
@@ -502,12 +272,118 @@ NOTE: this command works only if the overriding class
         (bsh-eval cmd))
     (error
      (message "%s" (error-message-string err)))))
+
+
+;; Required (at least for me) because indent-region does not seem to
+;; indent javadoc comments properly.  This function is used to indent
+;; the code inserted by jde-wiz-delegate.  (It might also be useful to
+;; indent the output of jde-wiz-implement-interface)
+(defun jde-indent-java-region (start end)
+  (save-excursion
+    (goto-char start)
+    (beginning-of-line)
+    (setq end (set-marker (make-marker) end))
+    (c-indent-line)
+    (while (< (point) end)
+      (c-indent-line)
+      (forward-line 1))))
+
+
+(defun jde-wiz-delegate (attribute-name)
+  "*Generate delegation methods for a given attribute.
+This method generates methods in the current buffer that delegate
+calls to methods of the given ATTRIBUTE of the current class.  For
+example, if the current buffer contains Java class A and there is
+an attribute in A named b of class type B, this method will
+generate in A, all the public methods of class B and delegate
+handling of those methods via attribute B."
+  (interactive
+   "sAttribute name: ")
+  (condition-case err
+      (let* ((start nil)
+             (class-name (or (jde-complete-get-qualified-name
+                              (jde-parse-declared-type-of attribute-name))
+                             (read-string (concat "Enter fully qualified class name of " 
+                                                  attribute-name ": "))))
+             (nl-brace-p
+	      (find 'before 
+		    (cdr (assoc 'defun-open c-hanging-braces-alist))))
+	     (code
+	      (bsh-eval-r
+	       (concat
+		"jde.wizards.DelegateFactory.makeDelegates(\""
+		attribute-name "\", \"" class-name "\", true, true, "
+		(if nl-brace-p "true" "false") ");"))))
+	(if code 
+	    (let ((required-imports
+		   (bsh-eval-r
+		    "jde.wizards.DelegateFactory.getImportedClasses();")))
+              (font-lock-mode -1)
+              (setq start (point))
+	      (insert code)
+              (jde-indent-java-region start (point))
+	      (if required-imports
+		  (jde-import-insert-imports-into-buffer required-imports))
+              (font-lock-mode)
+              )))	  
+    (error
+     (message "%s" (error-message-string err)))))
+
   
 (provide 'jde-wiz)
 
 ;; $Log: jde-wiz.el,v $
-;; Revision 1.1  2000/08/13 13:44:01  michaels
-;; Initial checkin.
+;; Revision 1.3  2001/02/12 05:38:26  paulk
+;; JDE 2.2.7
+;;
+;; Revision 1.38  2000/12/21 13:24:12  paulk
+;; Changed jde-wiz-insert-imports to jde-import-insert-imports to reflect recent repackaging scheme.
+;;
+;; Revision 1.37  2000/12/18 05:22:46  paulk
+;; *** empty log message ***
+;;
+;; Revision 1.36  2000/11/27 06:18:41  paulk
+;; Miscellaneous bug fixes and minor enhancements.
+;;
+;; Revision 1.35  2000/11/20 05:15:16  paulk
+;; Added jde-import-organize command. Moved all import-related code from
+;; jde-wiz.el to a new package named jde-import.el.
+;;
+;; Revision 1.34  2000/11/17 04:02:24  paulk
+;; jde-wiz-update-class-list now uses the current jde-global-classpath when building the class list. This eliminates the need to restart the beanshell when switching projects.
+;;
+;; Revision 1.33  2000/11/16 05:44:56  paulk
+;; Fixed problem in jde-sort-imports command. The problem was that jde-sort-imports temporarily defined sort-fold-case just before invoking sort-lines. Invoking sort-lines caused the sort package to be loaded. Since sort-fold-case is already defined, the sort package did not bother to define it. Then sort-lines returns to jde-sort-lines which proceeded to destroy the temporary copy of sort-fold-case. The fix is to have jde-sort-lines require the sort package before invoking sort-lines.
+;;
+;; Revision 1.32  2000/11/16 04:53:26  paulk
+;; Adds jde-wiz-kill-extra-imports command contributed by David Ponce.
+;;
+;; Revision 1.31  2000/10/25 04:35:20  paulk
+;; Updated sort import function to reflect new bovinator syntax.
+;;
+;; Revision 1.30  2000/10/08 12:55:39  paulk
+;; *** empty log message ***
+;;
+;; Revision 1.29  2000/09/24 08:42:52  paulk
+;; Now sorts import list after adding an import. To disable this feature, set jde-auto-sort-imports off. Thanks to "Jason Stell" <Jason.Stell@globalone.net>
+;;
+;; Revision 1.28  2000/08/03 04:54:11  paulk
+;; Restored use of the radio buttons by the import wizard. Somehow this functionality got deleted.
+;;
+;; Revision 1.27  2000/07/28 06:27:46  paulk
+;; Committing all modified files.
+;;
+;; Revision 1.26  2000/07/14 05:22:57  paulk
+;; Adds a delegation wizard contributed by Charles Hart.
+;;
+;; Revision 1.25  2000/07/13 07:18:24  paulk
+;; * You can now specify a list of packages to exclude from import
+;;   into a source file. See jde-wiz-import-excluded-packages for
+;;   more information. Thanks to "Jim Loverde" <loverde@str.com>
+;;   for this enhancement.
+;;
+;; * Changed name of jde-wiz-insert-excluded-packages-regexp to
+;;   jde-wiz-import-excluded-packages.
 ;;
 ;; Revision 1.23  2000/06/22 02:50:25  paulk
 ;; The import wizard dialog now uses radio buttons rather than check boxes to select
